local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local histogramColumns = 60
local componentsPerPage = 5

-- TODO: QueryAuctionItems (and related) are used in multiple places independently and seem to have a high likelihood of colliding -- badly.
--       There needs to be a consolidated place to capture the logic of querying the AH, including prioritization and cancelling of requests.
-- CONSIDER: As part of the above TODO, there's an opportunity to capture "Savage Leather Scraps" information "for free" when scanning for "Savage Leather".
--           This is much like the forced bucketing that happens for "Glyph of " and "Enchant of ".  Perhaps encapsulate the whole concept into the design
--           of the scanning subsystem such that multiple requests are done by "smart substrings" and extra data for "unintentional multiple hits" of an
--           "unintentionally broad" search (like Savage Leather / Savage Leather Scraps) becomes a good thing rather than a bad.
-- REVIEW: With the CONSIDER in place, a feature that automatically updates data for visible then "not yet visible" pages becomes much simpler to implement.

-- REVIEW: Why not just Hide() instead?
-- Using SetHeight(0) is invalid.  So we use something small to approximate the desired invisibility.
local widgetMinHeight = 0.01

-- REVIEW: Naming convention for file: 'reagent' to 'component'.

function Capitalist:CreateComponentUI()
	self:DebugEntry("CreateComponentUI")


	self.componentUIButton = CreateFrame("button", nil, AuctionFrameBrowse)
	local t = self.componentUIButton
	t:SetPoint("LEFT", self.auctionUIButton, "RIGHT", 0, 0)
	t:SetHeight(30)
	t:SetWidth(30)
	t:SetScript("OnClick", function() Capitalist:ToggleComponentUI() end)
	t:Show()

	-- REVIEW: The abyss crystal icon isn't a great generic component icon (with the shift from Enchanting-only)	
	--         BUT it might be a great icon for (proposed feature) enchanting-only component shopping.
	--         Same with scroll icon for products/posting UI.
	t.texture = t:CreateTexture()
	t.texture:SetAllPoints()
	t.texture:SetTexture("Interface\\Icons\\inv_enchant_abysscrystal")
	t.texture:SetDrawLayer("BACKGROUND")
	
	t.border = t:CreateTexture()
	t.border:SetAllPoints()
	t.border:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
	t.border:SetBlendMode("ADD")
	t.border:SetDrawLayer("BORDER")
	t.border:Hide()
	t:SetScript("OnEnter", function(self) self.border:Show() end)
	t:SetScript("OnLeave", function(self) if (not Capitalist.componentUI:IsVisible()) then self.border:Hide() end end)

	
	self.componentUI = CreateFrame("frame", nil, AuctionFrameBrowse)
	t = self.componentUI
	t:SetPoint("TOPLEFT", AuctionFrame, "TOPLEFT", 16, -101)
	t:SetPoint("BOTTOMRIGHT", AuctionFrame, "BOTTOMRIGHT", -7, 37)
	t:SetFrameStrata("HIGH")
	t:EnableMouse(true)
	t:Hide()
	
	t.background = t:CreateTexture(nil, "BACKGROUND")
	t.background:SetAllPoints()
	t.background:SetTexture(0,0,0)
	
	
	t.items = {}
	for i = 1, componentsPerPage do
		t.items[i] = CreateFrame("frame", nil, t)
		local item = t.items[i]
		item:SetWidth(500)
		item:SetHeight(52)
		item:SetPoint("TOPLEFT", t, "TOPLEFT", 0, 32-52*i)
		
		item.histogram = CreateFrame("frame", nil, item)
		local hist = item.histogram
		hist:SetWidth(242)
		hist:SetHeight(52)
		hist:SetPoint("TOPLEFT", 4, 0)
		
		hist.background = hist:CreateTexture(nil, "BACKGROUND")
		hist.background:SetAllPoints()
		hist.background:SetTexture(0.1, 0.1, 0.1)
		
		hist.segments = {}
		hist.segmentBackgrounds = {}
		for j = 1, histogramColumns do
			hist.segments[j] = hist:CreateTexture(nil, "OVERLAY")
			local s = hist.segments[j]
			s:SetWidth(4)
			s:SetHeight(10)
			s:SetPoint("BOTTOMLEFT", 4*(j-1)+1, 1)
			s:SetTexture(j/histogramColumns, 1-(j/histogramColumns), 0)
			
			hist.segmentBackgrounds[j] = hist:CreateTexture(nil, "BORDER")
			s = hist.segmentBackgrounds[j]
			s:SetWidth(4)
			s:SetHeight(50)
			s:SetPoint("BOTTOMLEFT", 4*(j-1)+1, 1)
			s:SetTexture(0,0,0)
		end
		
		
		item.name = item:CreateFontString(nil, nil, "GameFontNormal")
		item.name:SetWidth(175)
		item.name:SetPoint("TOPLEFT", 250, -10)
		item.name:SetText("")
		item.name:SetJustifyH("LEFT")
		
		item.scanButton = CreateFrame("button", nil, item, "UIPanelButtonTemplate")
		item.scanButton:SetWidth(60)
		item.scanButton:SetHeight(20)
		item.scanButton:SetText("scan")
		item.scanButton:SetPoint("TOPLEFT", item.name, "BOTTOMLEFT")
		item.scanButton:SetScript("OnClick", function() Capitalist:ScanComponent(i) end)
		
		item.info1 = item:CreateFontString(nil, nil, "GameFontNormal")
		item.info1:SetWidth(315)
		item.info1:SetPoint("TOPLEFT", 425, 0)
		item.info1:SetText("")
		item.info1:SetJustifyH("LEFT")
		
		item.info2 = item:CreateFontString(nil, nil, "GameFontNormal")
		item.info2:SetWidth(315)
		item.info2:SetPoint("TOPLEFT", item.info1, "BOTTOMLEFT")
		item.info2:SetText("")
		item.info2:SetJustifyH("LEFT")
		
		item.info3 = item:CreateFontString(nil, nil, "GameFontNormal")
		item.info3:SetWidth(315)
		item.info3:SetPoint("TOPLEFT", item.info2, "BOTTOMLEFT")
		item.info3:SetText("")
		item.info3:SetJustifyH("LEFT")
		
		item.prevStackButton = CreateFrame("button", nil, item, "UIPanelButtonTemplate")
		item.prevStackButton:SetWidth(15)
		item.prevStackButton:SetHeight(20)
		item.prevStackButton:SetText("<")
		item.prevStackButton:SetPoint("TOPLEFT", 730, 0)
		item.prevStackButton:SetScript("OnClick", function()
												local name = Capitalist.componentUI.items[i].name:GetText()
												local currentStack = Capitalist.componentCurrentStack[name]
												local count = Capitalist.componentScanData[name][currentStack].count
												local smallerStack
												local index
												
												for i = currentStack, 1, -1 do
													local size = Capitalist.componentScanData[name][i].count
													if (not smallerStack and size < count) then
														smallerStack = size
													end
													if (size == smallerStack) then
														index = i
													end
												end
												
												if (index) then
													Capitalist.componentCurrentStack[name] = index
													Capitalist:UpdateHistogram(i)
													Capitalist:SearchStack(name)
												end
											end)
		
		item.prevButton = CreateFrame("button", nil, item, "UIPanelButtonTemplate")
		item.prevButton:SetWidth(25)
		item.prevButton:SetHeight(20)
		item.prevButton:SetText("<<")
		item.prevButton:SetPoint("LEFT", item.prevStackButton, "RIGHT")
		item.prevButton:SetScript("OnClick", function()
												local name = Capitalist.componentUI.items[i].name:GetText()
												Capitalist.componentCurrentStack[name] = Capitalist.componentCurrentStack[name] - 1
												Capitalist:UpdateHistogram(i)
												Capitalist:SearchStack(name)
											end)

		item.nextButton = CreateFrame("button", nil, item, "UIPanelButtonTemplate")
		item.nextButton:SetWidth(25)
		item.nextButton:SetHeight(20)
		item.nextButton:SetText(">>")
		item.nextButton:SetPoint("LEFT", item.prevButton, "RIGHT")
		item.nextButton:SetScript("OnClick", function()
												local name = Capitalist.componentUI.items[i].name:GetText()
												Capitalist.componentCurrentStack[name] = Capitalist.componentCurrentStack[name] + 1
												Capitalist:UpdateHistogram(i)
												Capitalist:SearchStack(name)
											end)

		item.nextStackButton = CreateFrame("button", nil, item, "UIPanelButtonTemplate")
		item.nextStackButton:SetWidth(15)
		item.nextStackButton:SetHeight(20)
		item.nextStackButton:SetText(">")
		item.nextStackButton:SetPoint("LEFT", item.nextButton, "RIGHT")
		item.nextStackButton:SetScript("OnClick", function()
												local name = Capitalist.componentUI.items[i].name:GetText()
												local currentStack = Capitalist.componentCurrentStack[name]
												local count = Capitalist.componentScanData[name][currentStack].count
												
												for j = currentStack, #Capitalist.componentScanData[name] do
													local size = Capitalist.componentScanData[name][j].count
													if (size > count) then
														Capitalist.componentCurrentStack[name] = j
														Capitalist:UpdateHistogram(i)
														Capitalist:SearchStack(name)
														return
													end
												end
											end)
											
		item.buyButton = CreateFrame("button", nil, item, "UIPanelButtonTemplate")
		item.buyButton:SetWidth(80)
		item.buyButton:SetHeight(20)
		item.buyButton:SetText("buy stack")
		item.buyButton:SetPoint("TOPLEFT", item.prevStackButton, "BOTTOMLEFT")
		item.buyButton:SetScript("OnEnter", function()
												Capitalist:SearchStack(Capitalist.componentUI.items[i].name:GetText())
											end)

		item.buyButton:SetScript("OnClick", function()
												Capitalist:BuyStack(Capitalist.componentUI.items[i].name:GetText())
											end)
	end
	
	t.page = 0
	
	t.prevButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.prevButton:SetWidth(30)
	t.prevButton:SetHeight(20)
	t.prevButton:SetText("<<")
	t.prevButton:SetPoint("TOPLEFT", t.items[componentsPerPage], "BOTTOMLEFT", 250, 0)
	t.prevButton:SetScript("OnClick", function()
										local i = Capitalist.componentUI.page
										if (i > 0) then i = i - 1 end
										Capitalist.componentUI.page = i
										Capitalist.componentUI.nextButton:Enable()
										if (i == 0) then
											Capitalist.componentUI.prevButton:Disable()
										end
										Capitalist:UpdateComponentUI()
									  end)
	t.prevButton:Disable()
	
	t.nextButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.nextButton:SetWidth(30)
	t.nextButton:SetHeight(20)
	t.nextButton:SetText(">>")
	t.nextButton:SetPoint("LEFT", t.prevButton, "RIGHT", 10, 0)
	t.nextButton:SetScript("OnClick", function()
									Capitalist.componentUI.page = Capitalist.componentUI.page + 1
									Capitalist.componentUI.prevButton:Enable()
									Capitalist:UpdateComponentUI()
								  end)

	self:DebugExit("CreateComponentUI")
end

function Capitalist:ToggleComponentUI()
	self:DebugEntry("ToggleComponentUI")

	if (self.auctionUI:IsVisible()) then
		self:ToggleAuctionUI()
	end

	if (self.componentUI:IsVisible()) then
		self.componentUI:Hide()
		self.componentUIButton.border:Hide()
		
		BrowseQualitySort:Show()
		BrowseLevelSort:Show()
		BrowseDurationSort:Show()
		BrowseHighBidderSort:Show()
		BrowseCurrentBidSort:Show()
	else
		self.componentUI:Show()
		self.componentUIButton.border:Show()
		
		BrowseQualitySort:Hide()
		BrowseLevelSort:Hide()
		BrowseDurationSort:Hide()
		BrowseHighBidderSort:Hide()
		BrowseCurrentBidSort:Hide()
	end

	self:DebugExit("ToggleComponentUI")
end

function Capitalist:ScanComponent(index)
	self:DebugEntry("ScanComponent")

	local name = self.componentUI.items[index].name:GetText()
	
	if (self.updateHandler:IsVisible()) then
		self:Print("Action in progress!")
	else
		self.mode = name
		self.updateHandler:Show()
		
		QueryAuctionItems(name)
		self.page = 0
		
		self.componentScanData[name] = {}
		self.componentCurrentStack[name] = 1
		
		for i = 1, histogramColumns do
			self.componentUI.items[index].histogram.segments[i]:SetTexture(i/histogramColumns, 1-(i/histogramColumns), 0)
		end
	end

	self:DebugExit("ScanComponent")
end

-- REVIEW: Revisit with new Scan approach used for products?
function Capitalist:OnUpdateComponentScanning()
	self:DebugEntry("OnUpdateComponentScanning")
	if (CanSendAuctionQuery()) then
		self.page = self.page + 1
		self:ScanComponents()
	end
	self:DebugExit("OnUpdateComponentScanning")
end

function Capitalist:ScanComponents()
	self:DebugEntry("ScanComponents")

	-- REVIEW: Order of operations.  We kick off a new page query BEFORE processing the data from the current page?  Why?
	-- The revamped implementation in OnUpdateScanning() specifically avoids this via InAuctionItemListUpdate.
	QueryAuctionItems(self.mode,nil,nil,0,0,0,self.page,0,0,nil)
	
	for i = 1, 50 do
		local name, _, count, _, _, _, _, _, buyoutPrice, _, _, _, _  = GetAuctionItemInfo("list", i)
	
		if (name == nil) then
			self:ComponentScanFinished()
			return
		end

		-- REVIEW: Double-check QueryAuctionItems (and related) to see if there's a direct or indirect way to get exact string search instead of substring.
		if (name == self.mode) then
			if (buyoutPrice > 0) then
				-- REVIEW: Putting a trace here showed this getting called an absurd number of times while scanning a reagent.
				local copperPrice = self:GetItemCopperShoppingPrice(name)

				-- REVIEW: Capture this criteria in a helper function and consider providing settings for user control/tuning.
				if ((copperPrice == 0) or
					(buyoutPrice < copperPrice * count * 4)) then
					table.insert(self.componentScanData[name], {count = count, price = buyoutPrice / count, order = i + (self.page - 1) * 50})
				end
			end
		end
	end

	-- REVIEW: Double-check QueryAuctionItems (and related) to see if there's a way to get the results sorted so that we can stop after the price is above the 60th column price.

	-- REVIEW: Double-check QueryAuctionItems (and related) to see if there's a way to get the results sorted such that the SearchStack "re-search completely" model can be improved.
	
	for i = 1, componentsPerPage do
		-- REVIEW: This is a bit round-about.  Shouldn't the index be the key and the name be a value fetched from that key.
		--         Or put another way...It seems inefficient to walk through all (5) items doing a string compare to find which histogram to update.	
		if (self.componentUI.items[i].name:GetText() == self.mode) then
			local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");
			local progress = math.floor(histogramColumns * self.page * 50 / totalAuctions)
			
			-- TODO: This is inefficient.  It's updating all 60 (histogramColumns) for every iteration instead of just the ones uncovered.
			-- Better algorithm would clear them all to widgetMinHeight and bump the appropriate ones up to full height as it progresses.
			-- Probably not a huge issue as it stands, but might be significant if/when "auto-scanning" of components is implemented.

			for j = 1, progress do
				-- REVIEW: These hard-coded UI metrics (50) all over the places could use a home for their constants.
				self.componentUI.items[i].histogram.segments[j]:SetHeight(50)
			end
			
			for j = progress + 1, histogramColumns do
				self.componentUI.items[i].histogram.segments[j]:SetHeight(widgetMinHeight)
			end
		end
	end

	self:DebugExit("ScanComponents")
end

function Capitalist:ComponentScanFinished()
	self:DebugEntry("ComponentScanFinished")

	self.updateHandler:Hide()

	-- REVIEW: A setting to buy stacks biggest first vs. smallest first?
	table.sort(self.componentScanData[self.mode], function(a, b)
													if (a.price == b.price) then
														return a.count > b.count
													else
														return a.price < b.price
													end
												end)

	-- REVIEW: This is a bit round-about.  Shouldn't the index be the key and the name be a value fetched from that key.
	--         Or put another way...It seems inefficient to walk through all (5) items doing a string compare to find which histogram to update.	
	for i = 1, componentsPerPage do
		if (self.componentUI.items[i].name:GetText() == self.mode) then
			self:UpdateHistogram(i)
		end
	end

	self:DebugExit("ComponentScanFinished")
end

function Capitalist:UpdateComponentUI()
	self:DebugEntry("UpdateComponentUI")

	if (not self.componentScanData) then
		self.componentScanData = {}
		self.componentCurrentStack = {}
	end
	
	local t = self.componentUI
	local componentList = self:GetSortedVisibleComponents()

	for i = 1, componentsPerPage do
		local index = i + self.componentUI.page * componentsPerPage

		-- REVIEW: This seems too late.  It should really be checking "will the next page be too far" rather than "will this page be too far".
		--         This might explain the "empty last page" problem I was seeing sometimes.
		if (index > #componentList) then
			t.items[i]:Hide()
			t.nextButton:Disable()
		else
			t.items[i]:Show()
			t.items[i].name:SetText(componentList[index])

			self:UpdateHistogram(i)
		end
	end

	self:DebugExit("UpdateComponentUI")
end

-- REVIEW: Function needs a new name and maybe a banner explaining its role.
-- I believe it preps the hints about how much we have in stock, needed, etc.
function Capitalist:CreateComponentStorageString(component)
	self:DebugEntry("CreateComponentStorageString")

	local numInInv = 0
	local numInBank = 0
	if (self.db.profile.storage[realm][character].bank and
		self.db.profile.storage[realm][character].bank[component]) then
		numInBank = self.db.profile.storage[realm][character].bank[component]
	end
	if (self.db.profile.storage[realm][character].inventory and
		self.db.profile.storage[realm][character].inventory[component]) then
		numInInv = self.db.profile.storage[realm][character].inventory[component]
	end
	
	local numOtherChars = 0
	for player, data in pairs(self.db.profile.storage[realm]) do
		if (player ~= character) then
			local t = (data.inventory and data.inventory[component])
			if (t) then numOtherChars = numOtherChars + t end
			t = (data.bank and data.bank[component])
			if (t) then numOtherChars = numOtherChars + t end
		end
	end
	
	local sum = 0
	
	local str = ""
	if (numInInv) then
		str = "Inv: "..numInInv
		sum = sum + numInInv
	end
	if (numInBank) then
		if (str ~= "") then str = str..", " end
		str = str.."Bank: "..numInBank
		sum = sum + numInBank
	end
	if (numOtherChars > 0) then
		if (str ~= "") then str = str..", " end
		str = str.."Other: "..numOtherChars
		sum = sum + numOtherChars
	end
	
	local missing = self.requiredComponents[component]
	if (missing) then
		missing = missing - sum
		if (missing > 0) then
			if (str ~= "") then str = str..", " end
			str = str.."Missing: "..missing
		end
	end
	
	return self:DebugExit("CreateComponentStorageString", str)
end

function Capitalist:UpdateHistogram(index)
	self:DebugEntry("UpdateHistogram")
	
	local item = self.componentUI.items[index]
	local component = item.name:GetText()
	local data = self.componentScanData[component]
	if (not data) then
		for i = 1, histogramColumns do
			item.histogram.segments[i]:SetHeight(10)
			item.histogram.segments[i]:SetTexture(i/histogramColumns, 1-(i/histogramColumns), 0)
			item.histogram.segmentBackgrounds[i]:SetTexture(0, 0, 0)
		end
		item.info1:SetText(self:CreateComponentStorageString(component))
		item.info2:SetText("")
		item.info3:SetText("")
		
		item.prevStackButton:Hide()
		item.prevButton:Hide()
		item.nextButton:Hide()
		item.nextStackButton:Hide()
		item.buyButton:Hide()

		return self:DebugExit("UpdateHistogram")
	elseif (#data == 0) then
		for i = 1, histogramColumns do
			item.histogram.segments[i]:SetHeight(widgetMinHeight)
		end

		local storageString = self:CreateComponentStorageString(component)
		if (storageString ~= "") then
			item.info1:SetText("AH: -, "..storageString)
		else
			item.info1:SetText("AH: -")
		end
		item.info2:SetText("")
		item.info3:SetText("")
		
		item.prevStackButton:Hide()
		item.prevButton:Hide()
		item.nextButton:Hide()
		item.nextStackButton:Hide()
		item.buyButton:Hide()

		return self:DebugExit("UpdateHistogram")
	end

	local minPrice = data[1].price
	local maxPrice = data[#data].price
	local myPrice = self:GetItemCopperShoppingPrice(component)

	-- save for text display
	
	if (minPrice > myPrice) then minPrice = myPrice end
	if (maxPrice < myPrice) then maxPrice = myPrice end
	
	local width = (maxPrice - minPrice) / (histogramColumns - 1)

	-- REVIEW: bugfix for division by zero in the math.floor below
	if (width == 0) then
		width = 59
	end

	local segments = {}
	for i = 1, histogramColumns do segments[i] = 0 end
	
	local maxHeight = 0
	
	for i = 1, #data do
		local segment = math.floor((data[i].price - minPrice) / width) + 1
		segments[segment] = segments[segment] + data[i].count
		if (segments[segment] > maxHeight) then
			maxHeight = segments[segment]
		end
	end
	
	local myPriceSegment = math.floor((myPrice - minPrice) / width) + 1
	
	for i = 1, histogramColumns do
		local segment = item.histogram.segments[i]
		local background = item.histogram.segmentBackgrounds[i]
	
		segment:SetHeight((segments[i] * 50 / maxHeight) + widgetMinHeight)

		-- CONSIDER: Feature - batch purchase all missing components (as tabulated by restock) that are at/below buy price (or at/below cost with additional confirm).		
		-- CONSIDER: Feature - batch purchase (with multiprompting as required) of all at/below designated buy price. 
		-- TODO:     Feature change - Revisit Green/Yellow/Red histogram columns and the 4x rule -- in ScanComponents() --
		--           to account for the new concept of buy price vs. cost.  When the market is tight, it's often useful
		--           to buy within the (buy price, cost] range.

		-- REVIEW: If the selling price is above our buy price but less than the next segment...
		--         it currently shows up in the same column.  Consider changing it so that there
		--         is a hard break at exactly our buy price so that we never buy anything even
		--         slightly "too expensive".
		if (i == myPriceSegment) then
			background:SetTexture(0.1, 0.1, 0.1)
			segment:SetTexture(1, 1, 0)
		else
			background:SetTexture(0, 0, 0)
			if (i < myPriceSegment) then
				segment:SetTexture(0, 1, 0)
			else
				segment:SetTexture(1, 0, 0)
			end
		end
	end
	
	-- update texts
	
	local numItems = 0
	for i = 1, #data do numItems = numItems + data[i].count end

	-- REVIEW: Consolidate info1 string preparation, etc. into helper function(s).
	local storageString = self:CreateComponentStorageString(component)
	if (storageString ~= "") then
		item.info1:SetText("AH: "..numItems..", "..storageString)
	else
		item.info1:SetText("AH: "..numItems)
	end

	item.info2:SetText(self:GetMoneyText(realMinPrice, false, 3).." - "..self:GetMoneyText(realMaxPrice, false, 3)..
												" (own price: "..self:GetMoneyText(myPrice, false, 3)..")")

	local currentStack = self.componentCurrentStack[component]
	local currentStackSize = data[currentStack].count
	local currentStackPrice = currentStackSize * data[currentStack].price
	
	item.info3:SetText("stack #"..currentStack..": "..currentStackSize.."x "..self:GetMoneyText(data[currentStack].price, false, 3)..
												" ("	..self:GetMoneyText(currentStackPrice, false, 3)..")")
												
	-- REVIEW: Consolidate text and histogram bar coloration into helper function(s).
	if (data[currentStack].price < myPrice) then
		item.info3:SetTextColor(0, 1, 0)
	elseif (data[currentStack].price == myPrice) then
		item.info3:SetTextColor(1, 1, 0)	
	else
		item.info3:SetTextColor(1, 0, 0)
	end
	
	-- update buttons
	item.prevStackButton:Show()
	item.prevButton:Show()
	item.nextButton:Show()
	item.nextStackButton:Show()
	item.buyButton:Show()

	item.prevStackButton:Disable()
	item.nextStackButton:Disable()
	
	for i = currentStack - 1, 1, -1 do
		if (data[i].count < currentStackSize) then
			item.prevStackButton:Enable()
		end
	end
	
	for i = currentStack + 1, #data do
		if (data[i].count > currentStackSize) then
			item.nextStackButton:Enable()
		end
	end
		
	if (currentStack == 1) then
		item.prevButton:Disable()
	else
		item.prevButton:Enable()
	end
	
	if (currentStack == #data) then
		item.nextButton:Disable()
	else
		item.nextButton:Enable()
	end
	
	-- REVIEW: This appears to 'set the stage' for BuyStack by spinning to the appropriate page so that
	--         a subsequent call to BuyStack "is very likely" to have the desired buy-able already ready
	--         to go, and just has to find the index for it.  But...if you buy more than just the one
	--         then each buy can/might trigger a re-SearchStack-to-Query pipeline again to find the newly
	--         appropriate auction page.  It would be nice if there was some way to uniquely identify an
	--         auction ID in a programmatic way rather than the page-based viewport that the normal user
	--         auction UI exposes.  I wonder if Blizzard has improved that since this code was written...
	self:SearchStack(component)

	self:DebugExit("UpdateHistogram")
end

function Capitalist:BuyStack(component)
	self:DebugEntry("BuyStack")

	local index, cost = self:SearchStack(component)
	if (index) then
		PlaceAuctionBid("list", index, cost)

		-- REVIEW: This doesn't verify if the buy was accepted or not...also it doesn't handle the case of buying too fast (PlaceAuctionBid xN before the game can respond).
		local currentStack = self.componentCurrentStack[component]
		local data = self.componentScanData[component]
		local order = data[currentStack].order
		table.remove(data, currentStack)

		-- REVIEW: What is this correcting/protecting?
		if (currentStack > #data) then currentStack = #data end
		
		for k, v in pairs(data) do
			if (v.order > order) then
				data[k].order = data[k].order - 1
			end
		end
		
		for i = 1, componentsPerPage do
			-- REVIEW: This is a bit round-about.  Shouldn't the index be the key and the name be a value fetched from that key.
			--         Or put another way...It seems inefficient to walk through all (5) items doing a string compare to find which histogram to update.	
			if (self.componentUI.items[i].name:GetText() == component) then
				self:UpdateHistogram(i)
			end
		end
	else
		-- start search for stack only if doing nothing or scanning for another component
		if ((not self.updateHandler:IsVisible()) or
			((self.mode == "componentsearching") and (self.componentSearch ~= component))) then

			-- REVIEW: My instinct is that this isn't a clean "stop, change, restart" but rather haphazard.  Need to revisit with debugging scaffolding.
			self.componentSearchedPages = {}
			self.componentSearch = component
			self.mode = "componentsearching"
			self.updateHandler:Show()
		end
	end

	self:DebugExit("BuyStack")
end

function Capitalist:SearchStack(component)
	self:DebugEntry("SearchStack")

	local itemIndex
	for i = 1, componentsPerPage do
		self.componentUI.items[i].buyButton:SetText("search")
		-- REVIEW: This is a bit round-about.  Shouldn't the index be the key and the name be a value fetched from that key.
		--         Or put another way...It seems inefficient to walk through all (5) items doing a string compare to find which histogram to update.	
		if (self.componentUI.items[i].name:GetText() == component) then
			itemIndex = i
		end
	end

	-- REVIEW: It seems like this should only happen if settings are changed or pages are switched while processing
	--         In such circumstances, we should be aborting such 'background' activities...
	if (not itemIndex) then return end

	local currentStack = self.componentCurrentStack[component]
	local data = self.componentScanData[component]
	
	local stackCount = data[currentStack].count
	local stackPrice = data[currentStack].price
	local stackOrder = data[currentStack].order
	
	-- search the current list for this auction
	for i = 1, GetNumAuctionItems("list") do
		local name, _, count, _, _, _, _, _, buyoutPrice, _, _, _, _  = GetAuctionItemInfo("list", i)
		if ((name == component) and (count == stackCount) and (buyoutPrice / count == stackPrice)) then
			self.componentUI.items[itemIndex].buyButton:SetText("buy stack")
			if ((self.mode == "componentsearching") and (self.componentSearch == component)) then
				self.updateHandler:Hide()
			end

			self:DebugExit("SearchStack")
			return i, buyoutPrice
		end
	end
	
	-- if not found start the update handler to find it
	if (not self.updateHandler:IsVisible()) then
		self.componentSearchedPages = {}
		self.componentSearch = component
		self.mode = "componentsearching"
		self.updateHandler:Show()
		
		local page = math.ceil(stackOrder / 50) - 1
		self.page = page
		QueryAuctionItems(component,nil,nil,0,0,0,page,0,0,nil)
	end

	self:DebugExit("SearchStack")
end

function Capitalist:OnUpdateComponentSearching()
	self:DebugEntry("OnUpdateComponentSearching")

	if (CanSendAuctionQuery()) then
		if (not self:SearchStack(self.componentSearch)) then
			self.componentSearchedPages[self.page] = true
			
			if (self.page > 0) then
				self.page = self.page - 1
			else
				self.page = #self.componentSearchedPages + 1
				-- only search further if there are still auctions
				if (GetNumAuctionItems("list") < 50) then
					self.updateHandler:Hide()
				end
			end
			
			QueryAuctionItems(self.componentSearch,nil,nil,0,0,0,self.page,0,0,nil)
		end
	end

	self:DebugExit("OnUpdateComponentSearching")
end
